import { Subscription } from 'rxjs';
import { createProxyAndHandleRequests } from '@adminide-stack/extension-api';
import { CommandsRegistry } from '@vscode-alt/monaco-editor/esm/vs/platform/commands/common/commands.js';
import { SubscriptionMap } from '../common.js';

class ClientCommands {
  registry;
  subscriptions = new Subscription();
  registrations = new SubscriptionMap();
  proxy;
  _disposables = new Map();
  _generateCommandsDocumentationRegistration;
  constructor(connection, registry) {
    this.registry = registry;
    this.subscriptions.add(this.registrations);
    this.proxy = createProxyAndHandleRequests('commands', connection, this);
    this._generateCommandsDocumentationRegistration = CommandsRegistry.registerCommand('_generateCommandsDocumentation', () => this._generateCommandsDocumentation());
  }
  dispose() {
    this._disposables.forEach(value => value.dispose());
    this._disposables.clear();
    this._generateCommandsDocumentationRegistration.dispose();
  }
  _generateCommandsDocumentation() {
    return this.proxy.$getContributedCommandHandlerDescriptions().then(result => {
      // add local commands
      const commands = CommandsRegistry.getCommands();
      for (let id in commands) {
        let {
          description
        } = commands[id];
        if (description) {
          result[id] = description;
        }
      }
      // print all as markdown
      const all = [];
      for (let id in result) {
        all.push('`' + id + '` - ' + _generateMarkdown(result[id]));
      }
    });
  }
  $unregister(id) {
    this.registrations.remove(id);
  }
  $registerCommand(id, command) {
    this.registrations.add(id, this.registry.registerCommand({
      command,
      run: editor => {
        // https://microsoft.github.io/monaco-editor/api/interfaces/monaco.editor.iactiondescriptor.html
        // Methods -> run() | run(editor: ICodeEditor): void | Promise<void>
        return this.proxy.$executeContributedCommand(id, []);
      }
    }));
  }
  $executeCommand(command, args) {
    // const commandService = StaticServices.instantiationService.get(ICommandService) as any;
    // commandService.executeCommand('line', () => console.log('command'));
    return this.registry.executeCommand({
      command,
      arguments: args
    });
  }
  unsubscribe() {
    this.subscriptions.unsubscribe();
  }
  $getCommands() {
    return Promise.resolve(Object.keys(CommandsRegistry.getCommands()));
  }
}
//TODO ICommandHandlerDescription should be in description type
function _generateMarkdown(description) {
  if (typeof description === 'string') {
    return description;
  } else {
    let parts = [description.description];
    parts.push('\n\n');
    if (description.args) {
      for (let arg of description.args) {
        parts.push(`* _${arg.name}_ - ${arg.description || ''}\n`);
      }
    }
    if (description.returns) {
      parts.push(`* _(returns)_ - ${description.returns}`);
    }
    parts.push('\n\n');
    return parts.join('');
  }
}

export { ClientCommands };
//# sourceMappingURL=client-commads.js.map
