import { Subscription, Observable, startWith, tap } from 'rxjs';
import { createProxyAndHandleRequests } from '@adminide-stack/extension-api';

class ClientExtensions {
  subscriptions = new Subscription();
  proxy;
  /**
   * Implements the client side of extensions API.
   *
   * @param connection The connection to the extension host.
   * @param extensionRegistry An observable that emits the set of extensions that should be activated
   *                          upon subscription and whenever it changes.
   */
  constructor(connection, extensionRegistry) {
    this.proxy = createProxyAndHandleRequests('extensions', connection, this);
    // Wrap the `activeExtensions` (which is a Subscribable) in an Observable
    const activeExtensions$ = new Observable(observer => {
      // Check if activeExtensions has a subscribe method (indicating it's a subscribable)
      if (typeof extensionRegistry.activeExtensions.subscribe === 'function') {
        // Subscribe to the Subscribable and push values into the RxJS observer
        const sub = extensionRegistry.activeExtensions.subscribe({
          next: extensions => {
            observer.next(extensions);
          },
          error: err => {
            observer.error(err);
          },
          complete: () => {
            observer.complete();
          }
        });
        // Cleanup logic when RxJS subscriber unsubscribes
        return () => {
          sub.unsubscribe();
        };
      } else {
        // If `activeExtensions` doesn't have a `subscribe` method, handle accordingly
        observer.error('`activeExtensions` is not subscribable');
      }
    });
    // Subscribe to the activeExtensions$ observable
    this.subscriptions.add(activeExtensions$.pipe(startWith([]),
    // Emit empty array initially
    tap(e => console.log('registrate', e))).subscribe(extensions => {
      const toActivate = [...extensions]; // Create a new array to avoid mutation
      const toDeactivate = [];
      // Add logic for activating and deactivating extensions as necessary
      for (const x of toDeactivate) {
        this.proxy.$deactivateExtension(x.id).catch(err => {
          console.warn(`Error deactivating extension ${JSON.stringify(x.id)}:`, err);
        });
      }
      for (const x of toActivate) {
        this.proxy.$activateExtension(x.id, x.scriptURL).catch(err => console.error(err, `Error activating extension ${JSON.stringify(x.id)}:`));
      }
    }));
  }
  unsubscribe() {
    this.subscriptions.unsubscribe();
  }
}

export { ClientExtensions };
//# sourceMappingURL=client-extensions.js.map
